package dev.fyloz.trial.colorrecipesexplorer.service

import com.nhaarman.mockitokotlin2.*
import dev.fyloz.trial.colorrecipesexplorer.model.*
import dev.fyloz.trial.colorrecipesexplorer.repository.MixMaterialRepository
import org.junit.jupiter.api.Test
import kotlin.test.assertEquals
import kotlin.test.assertFalse
import kotlin.test.assertNotEquals
import kotlin.test.assertTrue

class MixMaterialServiceTest : AbstractModelServiceTest<MixMaterial, MixMaterialService, MixMaterialRepository>() {
    override val repository: MixMaterialRepository = mock()
    private val materialService: MaterialService = mock()
    override val service: MixMaterialService = spy(MixMaterialServiceImpl(repository, materialService))

    private val material: Material = material(id = 0L)
    override val entity: MixMaterial = mixMaterial(id = 0L, material = material, quantity = 1000f)
    override val anotherEntity: MixMaterial = mixMaterial(id = 1L, material = material)

    // existsByMaterial()

    @Test
    fun `existsByMaterial() returns true when a mix material with the given material exists`() {
        whenever(repository.existsByMaterial(material)).doReturn(true)

        val found = service.existsByMaterial(material)

        assertTrue(found)
    }

    @Test
    fun `existsByMaterial() returns false when no mix material with the given material exists`() {
        whenever(repository.existsByMaterial(material)).doReturn(false)

        val found = service.existsByMaterial(material)

        assertFalse(found)
    }

    // createFromMap()

    @Test
    fun `createFromMap() calls create() for each map entry`() {
        val mix = mix()
        val map = mapOf(
            1L to 1000f,
            2L to 2000f,
            5L to 5000f
        )
        val mixMaterials = map.map { mixMaterial(material = material(id = it.key), quantity = it.value) }

        doAnswer { mixMaterials.first { mixMaterial -> mixMaterial.material.id == it.arguments[1] } }
            .whenever(service).create(eq(mix), any(), any())

        val found = service.createFromMap(mix, map)

        assertEquals(mixMaterials, found)
    }

    // create()

    @Test
    fun `create() creates a mix material with the given mix, material and quantity`() {
        val mix = mix()
        val material = material(id = 0L)
        val quantity = 1000f
        val mixMaterial = mixMaterial(mix = mix, material = material, quantity = quantity)

        whenever(materialService.getById(material.id!!)).doReturn(material)

        val found = service.create(mix, material.id!!, quantity)

        assertEquals(mixMaterial, found)
    }

    // updateQuantity()

    @Test
    fun `updateQuantity() updates the given mix material with the given quantity`() {
        val quantity = 5000f
        assertNotEquals(quantity, entity.quantity, message = "Quantities must not be equals for this test to works")

        doAnswer { it.arguments[0] }.whenever(service).update(any())

        val found = service.updateQuantity(entity, quantity)

        assertEquals(found.quantity, quantity)
    }
}
